@@ -462,7 +462,7 @@ let external_defined_value ctx k =
 	Define.raw_defined_value ctx.defines (convert_define k)
 
 let reserved_flags = [
-	"true";"false";"null";"cross";"js";"lua";"neko";"flash";"php";"cpp";"cs";"java";"python";
+	"true";"false";"null";"cross";"js";"lua";"neko";"flash";"php";"cpp";"cs";"java";"python";"hl";"hlc";
 	"swc";"macro";"sys";"static";"utf16";"haxe";"haxe_ver"
 	]
 
@@ -971,7 +971,8 @@ let set_platform com pf file =
 	if com.platform <> Cross then failwith "Multiple targets";
 	init_platform com pf;
 	com.file <- file;
-	if (pf = Flash) && Path.file_extension file = "swc" then define com Define.Swc;
+	if (pf = Flash) && Path.file_extension file = "swc" then define com Define.Swc
+	else if (pf = Hl) && Path.file_extension file = "c" then define com Define.Hlc;
 	(* Set the source header, unless the user has set one already or the platform sets a custom one *)
 	if not (defined com Define.SourceHeader) && (pf <> Hl) then
 		define_value com Define.SourceHeader ("Generated by Haxe " ^ s_version_full)
