@@ -91,7 +91,7 @@ let null t p = mk (TConst TNull) t p
 
 let mk_mono() = TMono (!monomorph_create_ref ())
 
-let rec t_dynamic = TDynamic t_dynamic
+let t_dynamic = TDynamic None
 
 let mk_anon ?fields status =
 	let fields = match fields with Some fields -> fields | None -> PMap.empty in
@@ -135,6 +135,21 @@ let mk_class m path pos name_pos =
 		cl_descendants = [];
 	}
 
+let mk_typedef m path pos name_pos t =
+	{
+		t_path = path;
+		t_module = m;
+		t_pos = pos;
+		t_name_pos = name_pos;
+		t_private = false;
+		t_doc = None;
+		t_meta = [];
+		t_params = [];
+		t_using = [];
+		t_type = t;
+		t_restore = (fun () -> ());
+	}
+
 let module_extra file sign time kind policy =
 	{
 		m_file = Path.UniqueKey.create_lazy file;
@@ -144,9 +159,9 @@ let module_extra file sign time kind policy =
 			m_type_hints = [];
 			m_import_positions = PMap.empty;
 		};
-		m_dirty = None;
+		m_cache_state = MSGood;
 		m_added = 0;
-		m_mark = 0;
+		m_checked = 0;
 		m_time = time;
 		m_processed = 0;
 		m_deps = PMap.empty;
@@ -201,6 +216,7 @@ let null_abstract = {
 	a_meta = [];
 	a_params = [];
 	a_using = [];
+	a_restore = (fun () -> ());
 	a_ops = [];
 	a_unops = [];
 	a_impl = None;
@@ -212,11 +228,16 @@ let null_abstract = {
 	a_array = [];
 	a_read = None;
 	a_write = None;
+	a_call = None;
 	a_enum = false;
 }
 
-let add_dependency m mdep =
-	if m != null_module && m != mdep then m.m_extra.m_deps <- PMap.add mdep.m_id mdep m.m_extra.m_deps
+let add_dependency ?(skip_postprocess=false) m mdep =
+	if m != null_module && m != mdep then begin
+		m.m_extra.m_deps <- PMap.add mdep.m_id mdep m.m_extra.m_deps;
+		(* In case the module is cached, we'll have to run post-processing on it again (issue #10635) *)
+		if not skip_postprocess then m.m_extra.m_processed <- 0
+	end
 
 let arg_name (a,_) = a.v_name
 
@@ -273,8 +294,10 @@ let map loop t =
 		let ft = lazy_type f in
 		let ft2 = loop ft in
 		if ft == ft2 then t else ft2
-	| TDynamic t2 ->
-		if t == t2 then	t else TDynamic (loop t2)
+	| TDynamic None ->
+		t
+	| TDynamic (Some t2) ->
+		TDynamic (Some (loop t2))
 
 let iter loop t =
 	match t with
@@ -300,8 +323,10 @@ let iter loop t =
 	| TLazy f ->
 		let ft = lazy_type f in
 		loop ft
-	| TDynamic t2 ->
-		if t != t2 then	loop t2
+	| TDynamic None ->
+		()
+	| TDynamic (Some t2) ->
+		loop t2
 
 let duplicate t =
 	let monos = ref [] in
@@ -319,6 +344,16 @@ let duplicate t =
 	in
 	loop t
 
+let dynamify_monos t =
+	let rec loop t =
+		match t with
+		| TMono { tm_type = None } ->
+			t_dynamic
+		| _ ->
+			map loop t
+	in
+	loop t
+
 exception ApplyParamsRecursion
 
 (* substitute parameters with other types *)
@@ -329,8 +364,8 @@ let apply_params ?stack cparams params t =
 	let rec loop l1 l2 =
 		match l1, l2 with
 		| [] , [] -> []
-		| (x,TLazy f) :: l1, _ -> loop ((x,lazy_type f) :: l1) l2
-		| (_,t1) :: l1 , t2 :: l2 -> (t1,t2) :: loop l1 l2
+		| {ttp_type = TLazy f} as tp :: l1, _ -> loop ({tp with ttp_type = lazy_type f} :: l1) l2
+		| tp :: l1 , t2 :: l2 -> (tp.ttp_type,t2) :: loop l1 l2
 		| _ -> die "" __LOC__
 	in
 	let subst = loop cparams params in
@@ -427,14 +462,16 @@ let apply_params ?stack cparams params t =
 				t
 			else
 				ft2
-		| TDynamic t2 ->
-			if t == t2 then
-				t
-			else
-				TDynamic (loop t2)
+		| TDynamic None ->
+			t
+		| TDynamic (Some t2) ->
+			TDynamic (Some (loop t2))
 	in
 	loop t
 
+let apply_typedef td tl =
+	apply_params td.t_params tl td.t_type
+
 let monomorphs eparams t =
 	apply_params eparams (List.map (fun _ -> mk_mono()) eparams) t
 
@@ -462,7 +499,7 @@ let rec follow t =
 	| TLazy f ->
 		follow (lazy_type f)
 	| TType (t,tl) ->
-		follow (apply_params t.t_params tl t.t_type)
+		follow (apply_typedef t tl)
 	| TAbstract({a_path = [],"Null"},[t]) ->
 		follow t
 	| _ -> t
@@ -476,7 +513,7 @@ let follow_once t =
 	| TAbstract _ | TEnum _ | TInst _ | TFun _ | TAnon _ | TDynamic _ ->
 		t
 	| TType (t,tl) ->
-		apply_params t.t_params tl t.t_type
+		apply_typedef t tl
 	| TLazy f ->
 		lazy_type f
 
@@ -489,7 +526,7 @@ let rec follow_without_null t =
 	| TLazy f ->
 		follow_without_null (lazy_type f)
 	| TType (t,tl) ->
-		follow_without_null (apply_params t.t_params tl t.t_type)
+		follow_without_null (apply_typedef t tl)
 	| _ -> t
 
 let rec follow_without_type t =
@@ -521,7 +558,7 @@ let rec is_nullable ?(no_lazy=false) = function
 		| _ -> is_nullable (lazy_type f)
 		)
 	| TType (t,tl) ->
-		is_nullable ~no_lazy (apply_params t.t_params tl t.t_type)
+		is_nullable ~no_lazy (apply_typedef t tl)
 	| TFun _ ->
 		false
 (*
@@ -552,7 +589,7 @@ let rec is_null ?(no_lazy=false) = function
 		| _ -> is_null (lazy_type f)
 		)
 	| TType (t,tl) ->
-		is_null ~no_lazy (apply_params t.t_params tl t.t_type)
+		is_null ~no_lazy (apply_typedef t tl)
 	| _ ->
 		false
 
@@ -565,7 +602,7 @@ let rec is_explicit_null = function
 	| TLazy f ->
 		is_explicit_null (lazy_type f)
 	| TType (t,tl) ->
-		is_explicit_null (apply_params t.t_params tl t.t_type)
+		is_explicit_null (apply_typedef t tl)
 	| _ ->
 		false
 
@@ -592,11 +629,29 @@ let concat e1 e2 =
 	) in
 	mk e e2.etype (punion e1.epos e2.epos)
 
+let extract_param_type tp = tp.ttp_type
+let extract_param_types = List.map extract_param_type
+let extract_param_name tp = tp.ttp_name
+let lookup_param n l =
+	let rec loop l = match l with
+		| [] ->
+			raise Not_found
+		| tp :: l ->
+			if n = tp.ttp_name then tp.ttp_type else loop l
+	in
+	loop l
+
+let mk_type_param n t def = {
+	ttp_name = n;
+	ttp_type = t;
+	ttp_default = def;
+}
+
 let type_of_module_type = function
-	| TClassDecl c -> TInst (c,List.map snd c.cl_params)
-	| TEnumDecl e -> TEnum (e,List.map snd e.e_params)
-	| TTypeDecl t -> TType (t,List.map snd t.t_params)
-	| TAbstractDecl a -> TAbstract (a,List.map snd a.a_params)
+	| TClassDecl c -> TInst (c,extract_param_types c.cl_params)
+	| TEnumDecl e -> TEnum (e,extract_param_types e.e_params)
+	| TTypeDecl t -> TType (t,extract_param_types t.t_params)
+	| TAbstractDecl a -> TAbstract (a,extract_param_types a.a_params)
 
 let rec module_type_of_type = function
 	| TInst(c,_) -> TClassDecl c
@@ -612,8 +667,8 @@ let rec module_type_of_type = function
 		raise Exit
 
 let tconst_to_const = function
-	| TInt i -> Int (Int32.to_string i)
-	| TFloat s -> Float s
+	| TInt i -> Int (Int32.to_string i, None)
+	| TFloat s -> Float (s, None)
 	| TString s -> String(s,SDoubleQuotes)
 	| TBool b -> Ident (if b then "true" else "false")
 	| TNull -> Ident "null"
@@ -800,4 +855,4 @@ let type_has_meta t m =
 let var_extra params e = {
 	v_params = params;
 	v_expr = e;
-}
\ No newline at end of file
+}
