@@ -92,6 +92,8 @@ let encode_uv_error (e:Error.t) =
 	| `ENOTTY -> 75
 	| `EFTYPE -> 76
 	| `EILSEQ -> 77
+	| `EOVERFLOW -> 78
+	| `ESOCKTNOSUPPORT -> 79
 	)
 
 let decode_uv_error v : Error.t =
@@ -174,6 +176,8 @@ let decode_uv_error v : Error.t =
 	| 75 -> `ENOTTY
 	| 76 -> `EFTYPE
 	| 77 -> `EILSEQ
+	| 78 -> `EOVERFLOW
+	| 79 -> `ESOCKTNOSUPPORT
 	| _ -> unexpected_value v "eval.luv.UVError"
 
 let luv_exception e =
@@ -542,11 +546,20 @@ let uv_error_fields = [
 		Error.set_on_unhandled_exception (fun ex ->
 			let msg =
 				match ex with
-				| HaxeError.Error (Custom msg,_) ->
+				| HaxeError.Error (Custom msg,_,_) ->
 					(* Eval interpreter rethrows runtime exceptions as `Custom "Exception message\nException stack"` *)
 					(try fst (ExtString.String.split msg "\n")
 					with _ -> msg)
-				| HaxeError.Error (err,_) -> HaxeError.error_msg err
+				| HaxeError.Error (err,p,_) ->
+						(* TODO hook global error reporting *)
+						(match (extract_located (HaxeError.error_msg p err)) with
+						| [] -> ""
+						| (s,_) :: [] -> s
+						| (s,_) :: stack ->
+							List.fold_left (fun acc (s,p) ->
+								Printf.sprintf "%s%s\n" acc (Lexer.get_error_pos (Printf.sprintf "%s:%d: ") p)
+							) (s ^ "\n") stack
+						);
 				| _ -> Printexc.to_string ex
 			in
 			let e = create_haxe_exception ~stack:(get_ctx()).exception_stack msg in
@@ -1579,9 +1592,9 @@ module F = struct
 end
 
 let file_fields = [
-	"get_stdin", VHandle (HFile File.stdin);
-	"get_stdout", VHandle (HFile File.stdout);
-	"get_stderr", VHandle (HFile File.stderr);
+	"stdin", VHandle (HFile File.stdin);
+	"stdout", VHandle (HFile File.stdout);
+	"stderr", VHandle (HFile File.stderr);
 	"createRequest", vfun0 (fun() ->
 		VHandle (HFileRequest (File.Request.make()))
 	);
@@ -2141,6 +2154,10 @@ let env_fields = [
 		and value = decode_native_string v2 in
 		encode_unit_result (Env.setenv name ~value)
 	);
+	"unsetEnv", vfun1 (fun v ->
+		let name = decode_string v in
+		encode_unit_result (Env.unsetenv name)
+	);
 	"environ", vfun0 (fun() ->
 		let encode env =
 			let map =
@@ -2372,8 +2389,8 @@ let passwd_fields = [
 		encode_result (fun (p:Passwd.t) ->
 			encode_obj_s [
 				"username",encode_string p.username;
-				"uid",vint p.uid;
-				"gid",vint p.gid;
+				"uid",vint (Unsigned.ULong.to_int p.uid);
+				"gid",vint (Unsigned.ULong.to_int p.gid);
 				"shell",encode_nullable encode_string p.shell;
 				"homedir",vnative_string p.homedir;
 			]
