@@ -671,8 +671,14 @@ module LocalDce = struct
 				add_var_flag v VUsed;
 				(try expr (get_var_value ctx.graph v) with Not_found -> ());
 				begin match Ssa.get_reaching_def ctx.graph v with
-					| None -> use (get_var_origin ctx.graph v)
-					| Some v -> use v;
+					| None ->
+						(* We don't want to fully recurse for the origin variable because we don't care about its
+						   reaching definition (issue #10972). Simply marking it as being used should be sufficient. *)
+						let v' = get_var_origin ctx.graph v in
+						if not (is_used v') then
+							add_var_flag v' VUsed
+					| Some v ->
+						use v;
 				end
 			end
 		and expr e = match e.eexpr with
@@ -684,9 +690,8 @@ module LocalDce = struct
 			| _ ->
 				Type.iter expr e
 		in
-		let bb_marked = ref [] in
 		let rec mark bb =
-			bb_marked := bb :: !bb_marked;
+			add_block_flag bb BlockDce;
 			DynArray.iter expr bb.bb_el;
 			DynArray.iter expr bb.bb_phi;
 			terminator_iter expr bb.bb_terminator;
@@ -694,11 +699,11 @@ module LocalDce = struct
 				if not (has_flag edge FlagDce) then begin
 					edge.cfg_flags <- FlagDce :: edge.cfg_flags;
 					if not ctx.config.const_propagation || has_flag edge FlagExecutable then
-						mark edge.cfg_from;
+						mark edge.cfg_to;
 				end
-			) bb.bb_incoming
+			) bb.bb_outgoing
 		in
-		mark ctx.graph.g_exit;
+		mark ctx.graph.g_root;
 		let rec sweep e = match e.eexpr with
 			| TBinop(OpAssign,{eexpr = TLocal v},e2) | TVar(v,Some e2) when not (keep v) ->
 				if has_side_effect e2 then
@@ -710,10 +715,12 @@ module LocalDce = struct
 			| _ ->
 				Type.map_expr sweep e
 		in
-		List.iter (fun bb ->
-			dynarray_map sweep bb.bb_el;
-			bb.bb_terminator <- terminator_map sweep bb.bb_terminator;
-		) !bb_marked;
+		Graph.iter_dom_tree ctx.graph (fun bb ->
+			if has_block_flag bb BlockDce then begin
+				dynarray_map sweep bb.bb_el;
+				bb.bb_terminator <- terminator_map sweep bb.bb_terminator;
+			end
+		)
 end
 
 module Debug = struct
@@ -743,6 +750,8 @@ module Debug = struct
 			| BKFunctionBegin _ -> "<function-begin>\n"
 			| BKFunctionEnd -> "<function-end>\n"
 			| BKLoopHead -> "<loop-head>\n"
+			| BKCatch v -> Printf.sprintf "<catch %s<%i>>" v.v_name v.v_id
+			| BKException -> "<exc>"
 			| _ -> ""
 		in
 		Printf.fprintf ch "n%i [shape=box,label=\"%s%s\"];\n" bb.bb_id s_kind (s_escape s)
@@ -781,8 +790,7 @@ module Debug = struct
 			edge bb_then "then";
 			edge bb_else "else";
 			edge bb_next "next";
-		| SEWhile(bb_head,bb_body,bb_next,_) ->
-			edge bb_head "loop-head";
+		| SEWhile(bb_body,bb_next,_) ->
 			edge bb_body "loop-body";
 			edge bb_next "next";
 		| SEMerge bb_next ->
@@ -931,13 +939,20 @@ module Run = struct
 	open AnalyzerConfig
 	open Graph
 
-	let with_timer detailed s f =
-		let timer = Timer.timer (if detailed then "analyzer" :: s else ["analyzer"]) in
+	let with_timer level identifier s f =
+		let name = match level with
+			| 0 -> ["analyzer"]
+			| 1 -> "analyzer" :: s
+			| 2 when identifier = "" -> "analyzer" :: s
+			| 2 -> "analyzer" :: s @ [identifier]
+			| _ -> ["analyzer"] (* whatever *)
+		in
+		let timer = Timer.timer name in
 		let r = f() in
 		timer();
 		r
 
-	let create_analyzer_context com config e =
+	let create_analyzer_context com config identifier e =
 		let g = Graph.create e.etype e.epos in
 		let ctx = {
 			com = com;
@@ -946,12 +961,17 @@ module Run = struct
 			(* For CPP we want to use variable names which are "probably" not used by users in order to
 			   avoid problems with the debugger, see https://github.com/HaxeFoundation/hxcpp/issues/365 *)
 			temp_var_name = (match com.platform with Cpp -> "_hx_tmp" | _ -> "tmp");
+			with_timer = (fun s f ->
+				with_timer config.detail_times identifier s f
+			);
+			identifier = identifier;
 			entry = g.g_unreachable;
 			has_unbound = false;
 			loop_counter = 0;
 			loop_stack = [];
 			debug_exprs = [];
 			name_stack = [];
+			did_optimize = false;
 		} in
 		ctx
 
@@ -960,7 +980,7 @@ module Run = struct
 
 	let there actx e =
 		if actx.com.debug then add_debug_expr actx "initial" e;
-		let e = with_timer actx.config.detail_times ["->";"filter-apply"] (fun () -> TexprFilter.apply actx.com e) in
+		let e = actx.with_timer ["->";"filter-apply"] (fun () -> TexprFilter.apply actx.com e) in
 		if actx.com.debug then add_debug_expr actx "after filter-apply" e;
 		let tf,t,is_real_function = match e.eexpr with
 			| TFunction tf ->
@@ -972,18 +992,18 @@ module Run = struct
 				let tf = { tf_args = []; tf_type = t; tf_expr = e; } in
 				tf,tfun [] t,false
 		in
-		with_timer actx.config.detail_times ["->";"from-texpr"] (fun () -> AnalyzerTexprTransformer.from_tfunction actx tf t e.epos);
+		actx.with_timer ["->";"from-texpr"] (fun () -> AnalyzerTexprTransformer.from_tfunction actx tf t e.epos);
 		is_real_function
 
 	let back_again actx is_real_function =
-		let e = with_timer actx.config.detail_times ["<-";"to-texpr"] (fun () -> AnalyzerTexprTransformer.to_texpr actx) in
+		let e = actx.with_timer ["<-";"to-texpr"] (fun () -> AnalyzerTexprTransformer.to_texpr actx) in
 		if actx.com.debug then add_debug_expr actx "after to-texpr" e;
 		DynArray.iter (fun vi ->
 			vi.vi_var.v_extra <- vi.vi_extra;
 		) actx.graph.g_var_infos;
-		let e = if actx.config.fusion then with_timer actx.config.detail_times ["<-";"fusion"] (fun () -> Fusion.apply actx.com actx.config e) else e in
+		let e = if actx.config.fusion then actx.with_timer ["<-";"fusion"] (fun () -> Fusion.apply actx e) else e in
 		if actx.com.debug then add_debug_expr actx "after fusion" e;
-		let e = with_timer actx.config.detail_times ["<-";"cleanup"] (fun () -> Cleanup.apply actx.com e) in
+		let e = actx.with_timer ["<-";"cleanup"] (fun () -> Cleanup.apply actx.com e) in
 		if actx.com.debug then add_debug_expr actx "after cleanup" e;
 		let e = if is_real_function then
 			e
@@ -1037,26 +1057,27 @@ module Run = struct
 
 	let run_on_expr actx e =
 		let is_real_function = there actx e in
-		with_timer actx.config.detail_times ["->";"idom"] (fun () -> Graph.infer_immediate_dominators actx.graph);
-		with_timer actx.config.detail_times ["->";"infer_scopes"] (fun () -> Graph.infer_scopes actx.graph);
-		with_timer actx.config.detail_times ["->";"var writes"] (fun () -> Graph.infer_var_writes actx.graph);
+		actx.with_timer ["->";"idom"] (fun () -> Graph.infer_immediate_dominators actx.graph);
+		actx.with_timer ["->";"infer_scopes"] (fun () -> Graph.infer_scopes actx.graph);
+		actx.with_timer ["->";"var writes"] (fun () -> Graph.infer_var_writes actx.graph);
 		if actx.com.debug then Graph.check_integrity actx.graph;
 		if actx.config.optimize && not actx.has_unbound then begin
-			with_timer actx.config.detail_times ["optimize";"ssa-apply"] (fun () -> Ssa.apply actx);
-			if actx.config.const_propagation then with_timer actx.config.detail_times ["optimize";"const-propagation"] (fun () -> ConstPropagation.apply actx);
-			if actx.config.copy_propagation then with_timer actx.config.detail_times ["optimize";"copy-propagation"] (fun () -> CopyPropagation.apply actx);
-			with_timer actx.config.detail_times ["optimize";"local-dce"] (fun () -> LocalDce.apply actx);
+			actx.did_optimize <- true;
+			actx.with_timer ["optimize";"ssa-apply"] (fun () -> Ssa.apply actx);
+			if actx.config.const_propagation then actx.with_timer ["optimize";"const-propagation"] (fun () -> ConstPropagation.apply actx);
+			if actx.config.copy_propagation then actx.with_timer ["optimize";"copy-propagation"] (fun () -> CopyPropagation.apply actx);
+			actx.with_timer ["optimize";"local-dce"] (fun () -> LocalDce.apply actx);
 		end;
 		back_again actx is_real_function
 
-	let rec reduce_control_flow ctx e =
-		let e = Type.map_expr (reduce_control_flow ctx) e in
-		Optimizer.reduce_control_flow ctx e
+	let rec reduce_control_flow com e =
+		let e = Type.map_expr (reduce_control_flow com) e in
+		Optimizer.reduce_control_flow com e
 
-	let run_on_field ctx config c cf = match cf.cf_expr with
-		| Some e when not (is_ignored cf.cf_meta) && not (Typecore.is_removable_field ctx cf) ->
-			let config = update_config_from_meta ctx.Typecore.com config cf.cf_meta in
-			let actx = create_analyzer_context ctx.Typecore.com config e in
+	let run_on_field com config c cf = match cf.cf_expr with
+		| Some e when not (is_ignored cf.cf_meta) && not (Typecore.is_removable_field com cf) && not (has_class_field_flag cf CfPostProcessed) ->
+			let config = update_config_from_meta com config cf.cf_meta in
+			let actx = create_analyzer_context com config (Printf.sprintf "%s.%s" (s_type_path c.cl_path) cf.cf_name) e in
 			let debug() =
 				print_endline (Printf.sprintf "While analyzing %s.%s" (s_type_path c.cl_path) cf.cf_name);
 				List.iter (fun (s,e) ->
@@ -1067,33 +1088,35 @@ module Run = struct
 				Debug.dot_debug actx c cf;
 				print_endline (Printf.sprintf "dot graph written to %s" (String.concat "/" (Debug.get_dump_path actx c cf)));
 			in
+			let maybe_debug () = match config.debug_kind with
+				| DebugNone -> ()
+				| DebugDot -> Debug.dot_debug actx c cf;
+				| DebugFull -> debug()
+			in
 			let e = try
 				run_on_expr actx e
 			with
 			| Error.Error _ | Abort _ | Sys.Break as exc ->
+				maybe_debug();
 				raise exc
 			| exc ->
 				debug();
 				raise exc
 			in
-			let e = reduce_control_flow ctx e in
-			begin match config.debug_kind with
-				| DebugNone -> ()
-				| DebugDot -> Debug.dot_debug actx c cf;
-				| DebugFull -> debug()
-			end;
+			let e = reduce_control_flow com e in
+			maybe_debug();
 			cf.cf_expr <- Some e;
 		| _ -> ()
 
-	let run_on_field ctx config c cf =
-		run_on_field ctx config c cf;
-		List.iter (run_on_field ctx config c) cf.cf_overloads
+	let run_on_field com config c cf =
+		run_on_field com config c cf;
+		List.iter (run_on_field com config c) cf.cf_overloads
 
-	let run_on_class ctx config c =
-		let config = update_config_from_meta ctx.Typecore.com config c.cl_meta in
+	let run_on_class com config c =
+		let config = update_config_from_meta com config c.cl_meta in
 		let process_field stat cf = match cf.cf_kind with
 			| Var _ when not stat -> ()
-			| _ -> run_on_field ctx config c cf
+			| _ -> run_on_field com config c cf
 		in
 		List.iter (process_field false) c.cl_ordered_fields;
 		List.iter (process_field true) c.cl_ordered_statics;
@@ -1107,7 +1130,7 @@ module Run = struct
 			| Some e ->
 				let tf = { tf_args = []; tf_type = e.etype; tf_expr = e; } in
 				let e = mk (TFunction tf) (tfun [] e.etype) e.epos in
-				let actx = create_analyzer_context ctx.Typecore.com {config with optimize = false} e in
+				let actx = create_analyzer_context com {config with optimize = false} (Printf.sprintf "%s.__init__" (s_type_path c.cl_path)) e in
 				let e = run_on_expr actx e in
 				let e = match e.eexpr with
 					| TFunction tf -> tf.tf_expr
@@ -1116,29 +1139,28 @@ module Run = struct
 				c.cl_init <- Some e
 		end
 
-	let run_on_type ctx config t =
+	let run_on_type com config t =
 		match t with
 		| TClassDecl c when (is_ignored c.cl_meta) -> ()
-		| TClassDecl c -> run_on_class ctx config c
+		| TClassDecl c -> run_on_class com config c
 		| TEnumDecl _ -> ()
 		| TTypeDecl _ -> ()
 		| TAbstractDecl _ -> ()
 
-	let run_on_types ctx types =
-		let com = ctx.Typecore.com in
+	let run_on_types com types =
 		let config = get_base_config com in
-		with_timer config.detail_times ["other"] (fun () ->
+		with_timer config.detail_times "" ["other"] (fun () ->
 			if config.optimize && config.purity_inference then
-				with_timer config.detail_times ["optimize";"purity-inference"] (fun () -> Purity.infer com);
-			List.iter (run_on_type ctx config) types
+				with_timer config.detail_times "" ["optimize";"purity-inference"] (fun () -> Purity.infer com);
+			List.iter (run_on_type com config) types
 		)
 end
 ;;
-Typecore.analyzer_run_on_expr_ref := (fun com e ->
+Typecore.analyzer_run_on_expr_ref := (fun com identifier e ->
 	let config = AnalyzerConfig.get_base_config com in
 	(* We always want to optimize because const propagation might be required to obtain
 	   a constant expression for inline field initializations (see issue #4977). *)
 	let config = {config with AnalyzerConfig.optimize = true} in
-	let actx = Run.create_analyzer_context com config e in
+	let actx = Run.create_analyzer_context com config identifier e in
 	Run.run_on_expr actx e
 )
\ No newline at end of file
