@@ -236,7 +236,7 @@ module TexprFilter = struct
 			let e_if eo = mk (TIf(e_not,e_break,eo)) com.basic.tvoid p in
 			let rec map_continue e = match e.eexpr with
 				| TContinue ->
-					Texpr.duplicate_tvars (e_if (Some e))
+					Texpr.duplicate_tvars e_identity (e_if (Some e))
 				| TWhile _ | TFor _ ->
 					e
 				| _ ->
